#!/usr/bin/perl

# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: David Malcolm <dmalcolm@redhat.com>

my @build;
my @link;
my @execute;

my @args=@ARGV;

my $buildfail = 0;
my $linkfail = 0;
my $execfail = 0;

my $totalfail = 0;
my $totalresult = "FAIL";
my $test = "";

sub simplify_res {
        my ($res) = @_;
        
        $res =~ m/^([A-Z]+).*/; # remove spaces from end of string
        return $1;
}

sub test_res {
        my ($test, $expected, $actual, $message) = @_;
        my $smallfail = 0;
        my $totalfail = 0;
        #print "PROCESSING $message: $test: $expected -> $actual\n";
        
        if ($expected ne $actual) {
                print "$message: $test: $expected -> $actual\n";
                if (($actual ne "PASS") and ($actual ne "UNTESTED") and ($actual ne "SKIP")) {
                    $totalfail = 1; # If this happens, test FAILs
                }
        }
        if ( ($actual ne "PASS") and ($actual ne "UNTESTED") and ($actual ne "SKIP")) {
                $smallfail = 1; # this means that sth is wrong
                                # (but if it's wrong also in expected file, ti's ok ;-))
        }
        return ($smallfail, $totalfail);
}

# Expected results file
open (FILE,"$args[0]") || die "Could not open $args[0]";
while (<FILE>) {
        my $line = $_;

        if ($line =~ /(.*): build: (.*)/) {
                $build{$1} = simplify_res($2);
        }
        if ($line =~ /(.*): link: (.*)/) {
                $link{$1} = simplify_res($2);
        }
        if ($line =~ /(.*): execution: (.*)/) {
                $execute{$1} = simplify_res($2);
        }
}

# Actual results file
open (FILE2,"$args[1]") || die "Could not open $args[1]";
while (<FILE2>) {
        my $line = $_;
        my $tmpfail = 0;
        my $tmpfailtotal = 0;

        if ($line =~ /(.*): build: (.*)/) {
                ($tmpfail, $tmpfailtotal) = test_res($1, $build{$1}, simplify_res($2), "Build difference");
                $buildfail = $buildfail + $tmpfail;
        }
        if ($line =~ /(.*): link: (.*)/) {
                ($tmpfail, $tmpfailtotal) = test_res($1, $link{$1}, simplify_res($2), "Link difference");
                $linkfail = $linkfail + $tmpfail;
        }
        if ($line =~ /(.*): execution: (.*)/) {
                ($tmpfail, $tmpfailtotal) = test_res($1, $execute{$1}, simplify_res($2), "Execution difference");
                $execfail = $execfail + $tmpfail;
        }
        $totalfail = $totalfail + $tmpfailtotal;
        #$aaa = $buildfail + $linkfail + $execfail;
        #print "failures: $aaa, regressions: $totalfail\n";
}

my $sum = $buildfail + $linkfail + $execfail;
if ($totalfail == 0) {
  $totalresult = "PASS";
} else {
  $totalresult = "FAIL";
}
print "=================================\n";
print "Build failures: $buildfail\n";
print "Link failures: $linkfail\n";
print "Exec failures: $execfail\n";
print "Total failures: $sum\n";
print "Total regressions: $totalfail\n";
print "Overall result: $totalresult\n";
print "=================================\n\n";
