#!/bin/sh

# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: David Malcolm <dmalcolm@redhat.com>

# This script tests local machine using posixtestsuite
# (http://posixtest.sourceforge.net/) and creates "expected-..." file for results
# comaparation on test machines (expected- file is same as posixtestsuite's
# logfile, but there is tests group abberation on begin of each line)
#
# Excepted file name consists of: expected-1.5.1-FedoraCorerelease5Bordeaux-i386
#    1.5.1 - version of posixtestsuite package
#    FedoraCorerelease5Bordeaux - content of /etc/redhat-release free of all non alphanumeric characters
#    i386 - architecture
#
# Required files:
#    tests-groups-$PACKAGE_VER.txt - list of tests groups abberations for
#       posixtestsuite's run-tests script
#    tests-defunct-$PACKAGE_VER.txt - list of defunct tests, these are going
#       to be deleted before run-tests start

# Set required variables
export PACKAGE="posixtestsuite-1.5.1.tar.gz"
export DOWNLOAD_URL="http://easynews.dl.sourceforge.net/sourceforge/posixtest/"
export LDFLAGS="-D_XOPEN_SOURCE=600 -lpthread -lrt -lm"
export DIR=`pwd`
export ARCH=`uname -i`
export FAMILY=`cat /etc/redhat-release | tr -dc "[:alnum:]"` # for example "RedHatEnterpriseLinuxASrelease4NahantUpdate4"
PACKAGE_VER=`basename $PACKAGE .gz` # remove posible file extensions
PACKAGE_VER=`basename $PACKAGE_VER .bz2`
PACKAGE_VER=`basename $PACKAGE_VER .tar`
PACKAGE_VER=`basename $PACKAGE_VER .tgz`
PACKAGE_VER=`basename $PACKAGE_VER .zip`
export PACKAGE_VER=`echo $PACKAGE_VER | cut -d- -f2` # cut out version number
export FILE_EXPECTED_RESULTS=$DIR/expected-$PACKAGE_VER-$FAMILY-$ARCH
export FILE_TESTS_GROUPS=$DIR/tests-groups-$PACKAGE_VER.txt
export FILE_TESTS_DEFUNCT=$DIR/tests-defunct-$PACKAGE_VER.txt

# Prepare file for results
rm -f $FILE_EXPECTED_RESULTS
touch $FILE_EXPECTED_RESULTS

# Check for required files
if [ ! -r $FILE_TESTS_GROUPS -o \
     ! -r $FILE_TESTS_DEFUNCT -o \
     ! -w $FILE_EXPECTED_RESULTS ]; then
  echo "ERROR: Can't access required files"
  exit 0
fi

# Download and unpack the test and cd to it's directory
echo "***** Downloading and unpacking posixtestsuite *****"
wget --timestamping --quiet $DOWNLOAD_URL$PACKAGE
tar -zxf $DIR/$PACKAGE
cd posixtestsuite

# Set recomended (from posixtestsuite-1.5.1.tar.gz) compile ldflags
echo "***** Setting recomended LDFLAGS *****"
echo $LDFLAGS >> LDFLAGS

# Remove defunct tests
echo "***** Removeing defunct tests *****"
for tests_bad in `grep -ve "^#" $FILE_TESTS_DEFUNCT`; do
  bad_test=`echo $tests_bad | sed -e "s/\.\.//g"` # we don't want to del anything outside of posixtestsuite directory
  echo "deleting test $tests_bad"
  rm -f $tests_bad.*
done

# Run tests
echo "***** Running test *****"
for tests_group in `grep -ve "^#" $FILE_TESTS_GROUPS`; do
  echo "----- $tests_group tests -----"
  rm -f results-$tests_group
  touch results-$tests_group
  ./run_tests $tests_group &> results-$tests_group
  sed -e "s/^/$tests_group /" results-$tests_group >> $FILE_EXPECTED_RESULTS
done
