#!/bin/sh

# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: David Malcolm <dmalcolm@redhat.com>

# This script runs the test
#
# SYNOPSIS
#   make run
#       or
#   ./runtest.sh posixtestsuite-X.Y.Z.tar.gz

# Source the common test script helpers
. /usr/bin/rhts-environment.sh

export DIR=`pwd`
export ARCH=`uname -i`
export FAMILY=`cat /etc/redhat-release | tr -dc "[:alnum:]"` # for example "RedHatEnterpriseLinuxASrelease4NahantUpdate4"
export PACKAGE=$1 # "posixtestsuite-1.5.1.tar.gz"
export LDFLAGS="-D_XOPEN_SOURCE=600 -lpthread -lrt -lm"
PACKAGE_VER=`basename $PACKAGE .gz` # remove file extensions
PACKAGE_VER=`basename $PACKAGE_VER .bz2`
PACKAGE_VER=`basename $PACKAGE_VER .tar`
PACKAGE_VER=`basename $PACKAGE_VER .tgz`
PACKAGE_VER=`basename $PACKAGE_VER .zip`
export PACKAGE_VER=`echo $PACKAGE_VER | cut -d- -f2` # cut out version number
export FILE_EXPECTED_RESULTS=$DIR/expected-$PACKAGE_VER-$FAMILY-$ARCH # expected results for this $PACKAGE_VER, $FAMILY and $ARCH
export FILE_TESTS_GROUPS=$DIR/tests-groups-$PACKAGE_VER.txt
export FILE_TESTS_DEFUNCT=$DIR/tests-defunct-$PACKAGE_VER.txt

# Check for expected file for actual platform
if [ ! -r $FILE_EXPECTED_RESULTS ]; then
  echo "ERROR: Can't read expected file for this platform ($FILE_EXPECTED_RESULTS)" | tee -a $OUTPUTFILE
  report_result $TEST FAIL
  exit 1
fi

# Check for required files
if [ ! -r $FILE_TESTS_GROUPS -o ! -r $FILE_TESTS_DEFUNCT ]; then
  echo "ERROR: Can't access required files" | tee -a $OUTPUTFILE
  exit 2
fi

# First, unpack the test and cd to it's directory
mkdir -p /mnt/testarea
cd /mnt/testarea
tar -zxf $DIR/$PACKAGE
cd /mnt/testarea/posixtestsuite

# Add recomended (from posixtestsuite-1.5.1.tar.gz) compile ldflags
echo $LDFLAGS >> LDFLAGS

# Remove defunct tests
echo "***** Removeing defunct tests *****" | tee -a $OUTPUTFILE
for tests_bad in `grep -ve "^#" $FILE_TESTS_DEFUNCT`; do
  bad_test=`echo $tests_bad | sed -e "s/\.\.//g"` # we don't want to del anything outside of posixtestsuite directory
  echo "deleting test $tests_bad" | tee -a $OUTPUTFILE
  rm -f $tests_bad.*
done



# Then build and execute tests
echo "***** Begin run_tests output *****" | tee -a $OUTPUTFILE
for tests_group in `grep -ve "^#" $FILE_TESTS_GROUPS`; do
  echo "----- Begin $tests_group tests -----" | tee -a $OUTPUTFILE
  ./run_tests $tests_group > results-$tests_group 2>&1 # don't print this on screen
  sed -e "s/^/log $tests_group /" results-$tests_group | tee -a $OUTPUTFILE
  echo "----- End of $tests_group tests -----" | tee -a $OUTPUTFILE
done
echo "***** End of run_tests output *****" | tee -a $OUTPUTFILE



# Then post-process the results to find the regressions
RESULT="FAIL"
RESULT_ALL="PASS" # This is probably not good aproach
RESULT_SCORE=0

echo "***** Begin diffresult output *****" | tee -a $OUTPUTFILE
for tests_group in `grep -ve "^#" $FILE_TESTS_GROUPS`; do
  sed -e "s/^[A-Z]\+ //" results-$tests_group > results-$tests_group-clean
  sed -e "s/^[A-Z]\+ //" $FILE_EXPECTED_RESULTS > $FILE_EXPECTED_RESULTS-$tests_group
  
  echo "----- Begin diffresult output: $tests_group -----" | tee -a $OUTPUTFILE
  tmpfile=`mktemp`
  echo "Useing tmpfile: $tmpfile"
  perl $DIR/diffresults.pl $FILE_EXPECTED_RESULTS-$tests_group results-$tests_group-clean > $tmpfile
  RESULT=`cat $tmpfile | grep "Overall result" | cut -d" " -f3`
  SCORE=`cat $tmpfile | grep "Total regressions" | cut -d" " -f3`
  sed -e "s/^/$tests_group /" $tmpfile | tee -a $OUTPUTFILE
  echo "----- End of diffresult output: $tests_group -----" | tee -a $OUTPUTFILE

  # If there is one bad tests group, overall result is FAIL
  if [ "$RESULT" = "FAIL" ]; then
    RESULT_ALL="FAIL"
  fi
  # Sum of all SCOREs
  RESULT_SCORE=`expr $RESULT_SCORE + $SCORE`

  # Partial report
  report_result $TEST/$tests_group $RESULT $tmpfile $SCORE
  rm -f $tmpfile $FILE_EXPECTED_RESULTS-$tests_group results-$tests_group-clean # drop some now useless files
done
echo "***** End of diffresult output *****" | tee -a $OUTPUTFILE

# Report
report_result $TEST $RESULT_ALL $OUTPUTFILE $RESULT_SCORE

exit 0
